/*
  MongoDB database create script

  Note:
    It is not strictly necessary to pre create the database and
    collections in MongoDB. The database is automatically created when 
    first used (connected) and a collection is automatically created
    at the first insert request. However, this provides external 
    visibility for the "schema" which we use
*/

var dbName = 'ECDB_master';

var collList = [
    'COUNTER',
    'FILEDETAIL',
    'LOG',
    'EVENT',
    'ENDEAVOURJOB',
    'JOBSESSION',
    'JOBLOG',
    'ACCESSLOG',
    'NOTIFICATION',
    'REPORTINSTANCE',
    'REPORT',
    'RESOURCEPOOL',
    'ROLE',
    'USER',
    'ASSET',
    'BLOB',
    'IBMFCM',

    /* CHANGE LOG 2.4 */ 
    'APPLICATIONSERVER',
    'APPLICATIONCREDENTIAL',
    'LDAP',
    'TENANT',
    'RBACENTITY',
];

/* Set logging level */
db.adminCommand({setParameter:1, logLevel:1})

/* Removed database drop step. Should be done independently */
/* Get to our database */
db = db.getMongo().getDB(dbName);

/* Then, create all our collections */
for (var i=0; i<collList.length; i++) {
  print("Creating collection: "+collList[i]);
  db.createCollection(collList[i]);
}

/* Finally, create required indexes for our collections */
print("Creating indexes for FILEDETAIL collection...");
db.FILEDETAIL.ensureIndex({key : 1});
db.FILEDETAIL.ensureIndex({rkey : 1});
db.FILEDETAIL.ensureIndex({uniqueFilename : 1});
db.FILEDETAIL.ensureIndex({entityId : 1});
db.FILEDETAIL.ensureIndex({nodeId : 1});
db.FILEDETAIL.ensureIndex({fileSize : -1});
db.FILEDETAIL.ensureIndex({fileSizeUsed : -1});
db.FILEDETAIL.ensureIndex({fileExtension : 1});
db.FILEDETAIL.ensureIndex({mtime : 1});
db.FILEDETAIL.ensureIndex({atime : 1});
db.FILEDETAIL.ensureIndex({ctime : 1});
db.FILEDETAIL.ensureIndex({type : 1});
db.FILEDETAIL.ensureIndex({volumeId : 1});
db.FILEDETAIL.ensureIndex({snapshotId : 1});
db.FILEDETAIL.ensureIndex({parentPath : 1});
db.FILEDETAIL.ensureIndex({virtualMachineID : 1});
db.FILEDETAIL.ensureIndex({virtualServerId : 1});
db.FILEDETAIL.ensureIndex({dataStoreId : 1});
db.FILEDETAIL.ensureIndex({node : 1});

print("Creating indexes for LOG collection...");
/*********************************************************/
/* comment out some index key to reduce the memory usage */
/*********************************************************/
db.runCommand({"convertToCapped":"LOG", size:5000000000});
db.LOG.ensureIndex({type : 1});
db.LOG.ensureIndex({logTime : 1});

print("Creating indexes for JOBLOG collection...");
/*********************************************************/
/* comment out some index key to reduce the memory usage */
/*********************************************************/
db.JOBLOG.ensureIndex({logTime : 1});
db.JOBLOG.ensureIndex({entityId : 1});
db.JOBLOG.ensureIndex({logTime : 1, entityId : 1});
db.JOBLOG.ensureIndex({type : 1});
db.JOBLOG.ensureIndex({jobsessionId : 1});
db.JOBLOG.ensureIndex({logClass : 1}); 
db.JOBLOG.ensureIndex({level : 1}); 
db.JOBLOG.ensureIndex({expiresAt : 1}, {expireAfterSeconds : 0});

print("Creating indexes for ACCESSLOG collection...");
db.runCommand({"convertToCapped":"ACCESSLOG", size:500000000});
db.ACCESSLOG.ensureIndex({logTime : 1});
db.ACCESSLOG.ensureIndex({entityId : 1});
db.ACCESSLOG.ensureIndex({logTime : 1, entityId : 1});
db.ACCESSLOG.ensureIndex({type : 1});

print("Creating indexes for LOGMESSAGE collection...");
db.LOGMESSAGE.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for EVENT collection...");
/*********************************************************/
/* comment out some index key to reduce the memory usage */
/*********************************************************/
db.EVENT.ensureIndex({entityId : 1});
db.EVENT.ensureIndex({type : 1});
db.EVENT.ensureIndex({category : 1});
db.EVENT.ensureIndex({eventTime : -1});
db.EVENT.ensureIndex({expiresAt : 1}, {expireAfterSeconds : 0});

print("Creating indexes for NOTIFICATION collection...");
db.NOTIFICATION.ensureIndex({category : 1});
db.NOTIFICATION.ensureIndex({type : 1});
db.NOTIFICATION.ensureIndex({subType : 1});
db.NOTIFICATION.ensureIndex({notificationTime : -1});
db.NOTIFICATION.ensureIndex({name : 1});
db.NOTIFICATION.ensureIndex({dataSource : 1});
db.NOTIFICATION.ensureIndex({expiresAt : 1}, {expireAfterSeconds : 0});

print("Creating indexes for JOBSESSION collection...");
db.JOBSESSION.ensureIndex({type : 1, subType : 1});
db.JOBSESSION.ensureIndex({sessionId : 1, taskId : 1});
db.JOBSESSION.ensureIndex({start : 1, end : 1});
db.JOBSESSION.ensureIndex({jobName : 1});
db.JOBSESSION.ensureIndex({jobKey : 1});
db.JOBSESSION.ensureIndex({jobId : 1});
db.JOBSESSION.ensureIndex({status : 1});
db.JOBSESSION.ensureIndex({duration : 1});
db.JOBSESSION.ensureIndex({results : 1});
db.JOBSESSION.ensureIndex({index : 1});
db.JOBSESSION.ensureIndex({resourceId : 1});
db.JOBSESSION.ensureIndex({trackingId : 1});
db.JOBSESSION.ensureIndex({serviceId : 1});
db.JOBSESSION.ensureIndex({description : 1});
/* CHANGE LOG 2.4 */
db.JOBSESSION.ensureIndex({displayName : 1});
/* CHANGE LOG 2.5.1 */
db.JOBSESSION.ensureIndex({nextAction : 1});


print("Creating indexes for ENDEAVOURJOB collection...");
db.ENDEAVOURJOB.ensureIndex({name : 1});
db.ENDEAVOURJOB.ensureIndex({jobKey : 1});
db.ENDEAVOURJOB.ensureIndex({entityId : 1, nextRun : 1});
db.ENDEAVOURJOB.ensureIndex({type : 1});
db.ENDEAVOURJOB.ensureIndex({subType : 1});
db.ENDEAVOURJOB.ensureIndex({serviceId : 1});
db.ENDEAVOURJOB.ensureIndex({lastRun : 1});
db.ENDEAVOURJOB.ensureIndex({nextRun : 1});
db.ENDEAVOURJOB.ensureIndex({status : 1});
db.ENDEAVOURJOB.ensureIndex({lastSessionStatus : 1});
db.ENDEAVOURJOB.ensureIndex({lastSessionDuration : 1});

print("Creating indexes for SCHEDULE collection...");
db.SCHEDULE.ensureIndex({entityId : 1});
db.SCHEDULE.ensureIndex({name : 1});
db.SCHEDULE.ensureIndex({category : 1});
db.SCHEDULE.ensureIndex({type : 1});

print("Creating indexes for ENDEAVOURTIRGGERJOB collection...");
db.ENDEAVOURTRIGGERJOB.ensureIndex({name : 1});
db.ENDEAVOURTRIGGERJOB.ensureIndex({entityId : 1});
db.ENDEAVOURTRIGGERJOB.ensureIndex({category : 1});
db.ENDEAVOURTRIGGERJOB.ensureIndex({triggerId : 1});
db.ENDEAVOURTRIGGERJOB.ensureIndex({jobId : 1});

print("Creating indexes for USER collection...");
db.USER.ensureIndex({entityId : 1}, {unique : true});
/* CHANGE LOG 2.4 */
db.USER.ensureIndex({tenantId : 1, name : 1}, {unique : true});

print("Creating indexes for RESOURCEPOOL collection...");
db.RESOURCEPOOL.ensureIndex({entityId : 1}, {unique : true});
db.RESOURCEPOOL.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for ROLE collection...");
db.ROLE.ensureIndex({entityId : 1}, {unique : true});
db.ROLE.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for DPX collection...");
db.DPX.ensureIndex({entityId : 1}, {unique : true});
db.DPX.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for HYPERV collection...");
db.HYPERVISOR.ensureIndex({entityId : 1}, {unique : true});
db.HYPERVISOR.ensureIndex({name : 1}, {unique : true});
db.HYPERVISOR.ensureIndex({type : 1});

print("Creating indexes for IBMSVC collection...");
db.IBMSVC.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for EMCVNX collection...");
db.EMCVNX.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for NETAPP collection...");
db.NETAPP.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for VCLOUD collection...");
db.VCLOUD.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for VSPHERE collection...");
db.VSPHERE.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for SMTP collection...");
db.SMTP.dropIndex('name_1');
db.SMTP.ensureIndex({name : 1}, {unique : false});

print("Creating indexes for ENDEAVOURPOLICY collection...");
/* CHANGE LOG 2.4 */
db.ENDEAVOURPOLICY.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.ENDEAVOURPOLICY.ensureIndex({tenantId : 1, name : 1, serviceId : 1}, {unique : true});

print("Creating indexes for ENDEAVOURJOB collection...");
/* CHANGE LOG 2.4 */
db.ENDEAVOURJOB.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.ENDEAVOURJOB.ensureIndex({tenantId : 1, name : 1, serviceId : 1}, {unique : true});
db.ENDEAVOURJOB.ensureIndex({displayName : 1});

print("Creating indexes for ENDEAVOURTASK collection...");
db.ENDEAVOURTASK.ensureIndex({name : 1, type : 1}, {unique : false});

print("Creating indexes for REPORTINSTANCE collection...");
db.REPORTINSTANCE.ensureIndex({name : 1}, {unique : true});
db.REPORTINSTANCE.ensureIndex({fileDate : 1}, {unique : false});

print("Creating indexes for REPORT collection...");
db.REPORT.ensureIndex({entityId : 1}, {unique :  true});
db.REPORT.ensureIndex({name : 1}, {unique : true});
db.REPORT.ensureIndex({parentReportId : 1},{unique : false});

print("Creating indexes for ASSET collection...");
db.ASSET.ensureIndex({entityId : 1}, {unique :  true});
db.ASSET.ensureIndex({name : 1}, {unique : false});
db.ASSET.ensureIndex({category : 1}, {unique : false});
db.ASSET.ensureIndex({subCategory : 1}, {unique : false});
db.ASSET.ensureIndex({product : 1}, {unique : false});
db.ASSET.ensureIndex({version : 1}, {unique : false});
db.ASSET.ensureIndex({minProductVersion : 1}, {unique : false});
db.ASSET.ensureIndex({maxProductVersion : 1}, {unique : false});

print("Creating indexes for BLOB collection...");
db.BLOB.ensureIndex({entityId : 1}, {unique :  true});
db.BLOB.ensureIndex({name : 1}, {unique : false});


print("Creating indexes for IBMFCM collection...");
db.IBMFCM.ensureIndex({entityId : 1}, {unique :  true});
db.IBMFCM.ensureIndex({name : 1}, {unique : true});
db.IBMFCM.ensureIndex({uniqueId : 1}, {unique : true});
db.IBMFCM.ensureIndex({siteId : 1}, {unique : false});

print("Creating indexes for SITE collection...");
db.SITE.ensureIndex({name : 1}, {unique : true});

print("Creating indexes for STORAGEPROFILE collection...");
db.STORAGEPROFILE.ensureIndex({name : 1, type: 1 }, {unique : true});


print("Creating indexes for VMCREDENTIAL collection...");
db.VMCREDENTIAL.ensureIndex({entityId : 1}, {unique :  true});

print("Creating indexes for APPLICATIONSERVER collection...");
db.APPLICATIONSERVER.ensureIndex({entityId : 1}, {unique :  true});

print("Creating indexes for APPLICATIONCREDENTIAL collection...");
db.APPLICATIONCREDENTIAL.ensureIndex({entityId : 1}, {unique :  true});

print("Creating indexes for IBMXIV collection...");
db.IBMXIV.ensureIndex({name : 1}, {unique : true});
db.COUNTER.insert({_id:"IBMXIV", "lastSeqNum" : 1000 });

/* CHANGE LOG 2.2 */
/* from load.js */
db.COUNTER.insert({_id:"IBMFCM", "lastSeqNum" : 1000});

db.COUNTER.insert({_id:"IBMSVC", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"ENDEAVOURTRIGGERJOB", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"SECURITYEVENT", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"SCHEDULE", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"USER", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"ROLE", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"RESOURCEPOOL", "lastSeqNum" : 1000 });

/* CHANGE LOG 2.4 */
print("Creating indexes for TENANT collection...");
db.TENANT.ensureIndex({entityId : 1}, {unique :  true});
db.TENANT.ensureIndex({name : 1}, {unique :  true});

print("Creating indexes for RBACENTITY collection...");
db.RBACENTITY.ensureIndex({name : 1}, {unique :  true});

db.COUNTER.insert({_id:"APPLICATIONSERVER", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"APPLICATIONCREDENTIAL", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"TENANT", "lastSeqNum" : 1001 });

print("Updating indexes for LDAP collection...");
db.LDAP.dropIndex({name : 1});
db.LDAP.ensureIndex({tenantId : 1}, {unique : true});
db.LDAP.ensureIndex({entityId : 1}, {unique :  true});
db.USER.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.LDAP.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.COUNTER.insert({_id:"LDAP", "lastSeqNum" : 1010 });

print("Creating indexes for CONSTANT collection...");
db.CONSTANT.ensureIndex({name : 1, type : 1, constName : 1}, {unique : true});
db.CONSTANT.ensureIndex({entityId : 1}, {unique :  true});
db.COUNTER.insert({_id:"CONSTANT", "lastSeqNum" : 1000 });

/* CHANGE LOG 2.5 */
print("Updating indexes for EVENT collection...");
db.EVENT.ensureIndex({userId : 1, user : 1});

print("Updating indexes for ACCESSLOG collection...");
db.ACCESSLOG.ensureIndex({userId : 1, user : 1});

/* CHANGE LOG 2.5.1 */
print("Renaming ECXALL resource pool...");
db.RESOURCEPOOL.update({"name": "ECXALL"}, {$set: {"name": "All Resources"}})

/* CHANGE LOG 2.5.2 */
print("Updating indexes for ENDEAVOURPOLICY collection...");
db.ENDEAVOURPOLICY.ensureIndex({displayName : 1});
db.ENDEAVOURPOLICY.ensureIndex({comment : 1});
db.ENDEAVOURPOLICY.ensureIndex({creationTime : 1});
db.ENDEAVOURPOLICY.ensureIndex({lastUpdated : 1});
db.ENDEAVOURPOLICY.ensureIndex({policyKey : 1});

/* CHANGE LOG 2.6 */
print("Updating indexes for ENDEAVOURPOLICY collection...");
db.ENDEAVOURPOLICY.ensureIndex({applicationType : 1});
db.ENDEAVOURPOLICY.ensureIndex({subType : 1});

print("Updating indexes for JOBSESSION collection...");
db.JOBSESSION.ensureIndex({sessionKey : 1, taskKey : 1}, {unique : true, sparse: true});

print("Updating ENDEAVOURJOB collection...");
db.ENDEAVOURJOB.update({"type":"protection", "actions.actionType": "START"},{$set:{"actions.$.parameters.actionname.promptText":"SLA Policy"}}, {multi: true});

/* CHANGE LOG 3.0 */
 
dir='/opt/ECX/tools/dbScripts'
cd(dir)
pwd=pwd()
print('Current directory is :' + pwd)

print("Updating site to primary...");
db.SITE.update({entityId:1000}, {"$set": {"name": "Primary", "description": "Primary Site"}});

/* Storage Profiles */

print("Updating STORAGEPROFILe counters  ...");
/* 2000 - 2100 - reserved for canned entries */
db.COUNTER.update({ "_id" : "STORAGEPROFILE"}, {"lastSeqNum" : 2100}, {upsert:true});
db.COUNTER.update({ "_id" : "SUBPOLICY"}, {"lastSeqNum" : 2100}, {upsert:true});
var storageprofilenames = ["./GoldSLA.json", "./SilverSLA.json", "./BronzeSLA.json"];
var storageprofiles = [];

for (i=0; i < storageprofilenames.length; i++) {
    storageprofiles.push(cat(storageprofilenames[i]));

}
for (i=0; i < storageprofiles.length; i++) {
    var x = JSON.parse(storageprofiles[i]);
    var d = new Date();
    x["spec"].subpolicy[0].trigger.activateDate = d.getTime()
    print("Inserting canned storageprofile:  " + x["name"]);
    db.STORAGEPROFILE.update({name : x["name"]}, {$set:x}, {upsert:true});
}


/* ### Identity ### */
print("Updating IDENTITY counters  ...");
db.COUNTER.update({ "_id" : "IDENTITYUSER"}, {"lastSeqNum" : 2100}, {upsert:true});
var identityfilenames = [];
var identities = [];

for (i=0; i < identityfilenames.length; i++) {
    identities.push(cat(identityfilenames[i]));

}
for (i=0; i < identities.length; i++) {
    var x = JSON.parse(identities[i]);
    print("Inserting canned identity:  " + x["name"]);
    db.IDENTITYUSER.update({name : x["name"]}, {$set:x}, {upsert:true});
}

print("Updating STORAGECOMMON counters  ...");
db.COUNTER.update({ "_id" : "STORAGECOMMON"}, {"lastSeqNum" : 2100}, {upsert:true});

print("Inserting Secondary site  ...");
db.SITE.insert({ "entityId" : NumberInt(2000), "name" : 'Secondary', "description" : 'secondary site', "defaultSite" : false });
print("Updating SITE counters  ...");
db.COUNTER.update({ "_id" : "SITE"}, {"lastSeqNum" : 2100}, {upsert:true});

print("MongoDB database create completed");
